﻿Ext.define('App.Control.Kivii.Finances.Payment.Accept.Form', {
    extend: 'System.Control.EntityForm',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.Control.Kivii.Finances.Payment.Accept.Form',
    // requires:['App.Control.ComboSelector'],
    autoScroll: true,
    layout: 'responsivecolumn',
    defaults: {
        xtype: 'textfield',
        labelWidth: 60,
        labelAlign: 'right',
        responsiveCls: 'big-50 small-100',
    },
    viewModel: {
        data: {
            ButtonText: '新增',
            ButtonIcon: 'x-fa fa-plus',
            entity: {
                Metadata: {},
                Kvid: null,
                Type: null,
                // OwnerKvid:KiviiContext.CurrentMember.OrganizationKvid,
                // OwnerName:KiviiContext.CurrentMember.OrganizationName,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                Currency: 'CNY'
            }
        }
    },
    tbar: ['->', {
        handler: '_onCreate',
        bind: {
            text: '{ButtonText}',
            iconCls: '{ButtonIcon}',
            disabled: '{!formIsValid}'
        }
    }],
    items: [{
        fieldLabel: '付款单位',
        responsiveCls: 'big-60 small-100',
        allowBlank: false,
        bind: '{entity.PayerName}'
    }, {
        xtype: 'numberfield',
        fieldLabel: '发生金额',
        emptyText: '请输入数字',
        hideTrigger: true,
        responsiveCls: 'big-40 small-100',
        reference: 'amount',
        minValue: 0,
        decimalPrecision: 2,
        bind: {
            value: '{entity.Amount}'
        },
        allowBlank: false
    }, {
        xtype: 'datefield',
        fieldLabel: '操作日期',
        // format: 'Y-m-d H:i',
        editable: false,
        allowBlank: false,
        responsiveCls: 'big-60 small-100',
        bind: '{entity.OperateTime}'
    }, {
        xtype: 'combo',
        fieldLabel: '类&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;型',
        allowBlank: false,
        responsiveCls: 'big-40 small-100',
        queryMode: 'local',
        displayField: 'name',
        valueField: 'type',
        editable: false,
        reference: 'PaymentType',
        filterPickList: true,
        store: {
            fields: ['name', 'type'],
            data: [{
                name: '银行到账',
                type: 'Bank'
            }, {
                name: '现金',
                type: 'Cash'
            }, {
                name: '刷卡',
                type: 'Pos'
            }]
        },
        bind: '{entity.Type}'
    }, {
        fieldLabel: '付款账户',
        responsiveCls: 'big-60 small-100',
        bind: '{entity.PayerAccountOwnerName}'
    },{
        fieldLabel: '银行卡号',
        responsiveCls: 'big-40 small-100',
        bind: '{entity.PayerAccountSerialNumber}'
    },{
        fieldLabel: '付款银行',
        responsiveCls: 'big-60 small-100',
        bind: '{entity.PayerAccountName}'
    },{
        xtype: 'comboSelector',
        fieldLabel: '所属范围',
        responsiveCls: 'big-40 small-100',
        dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
        editable: false,
        // allowBlank: false,
        keyField: 'Kvid',
        valueField: 'FullName',
        bind: {
            key:'{entity.OwnerKvid}',
            value: '{entity.OwnerName}'
        },
        listeners: {
            select: '_onSelectName'
        }
    },{
        fieldLabel: '摘&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;要',
        responsiveCls: 'big-100 small-100',
        bind: '{entity.Summary}'
    }, {
        xtype: 'textarea',
        fieldLabel: '备&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;注',
        responsiveCls: 'big-100 small-100',
        bind: '{entity.Remark}'
    }],
    setOwner: function(_entity, _ctrl) {
        let _view = this;
        let _vm = _view.getViewModel();
        let _refs = _view.getReferences();
        _refs.PaymentType.setReadOnly(true);
        _vm.set('entity', _entity);
        _vm.set('ButtonText', _ctrl.text);
        _vm.set('ButtonIcon', _ctrl.iconCls);
    },
    controller: {
        _onCreate: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _data = _vm.data.entity;
            if (_data.Type === null) return;
            _controller._acceptData(_ctrl,_data);
        },
        _acceptData:function(_ctrl,_item){
        	let _controller = this;
            let _view = _controller.getView();
        	_ctrl.setDisabled(true);
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Accept.json',
                jsonData: {
                    Item: _item,
                    OwnerType:_item.OwnerType
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast("新增成功!");
                    _view.fireEvent('entityCreated', _view, _rtns.Results);
                    _ctrl.setDisabled(false);
                },
                failure: function(_response) {
                    Ext.toast("新增失败!");
                    _ctrl.setDisabled(false);
                }
            });
        }
    }
});