﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Statistic.Invoice', {
    extend: 'System.Control.EntityList',
    xtype: 'App.View.Kivii.Finances.Statistic.Invoice',
    entityType: 'Kivii.Finances.Entities.Invoice',
    requires: ['System.Control.ComboSelector', 'App.Control.DownloadField'],
    dataSource: '/Restful/Kivii.Finances.Entities.Invoice/Statistic.json',
    globalFilter: {},
    filter: {
        BeginTime: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
        EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
        Offseted: false,
        Payed: true
    },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar: true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer: false,
    header: true,
    layout: 'fit',
    title: '发票统计',
    iconCls: 'x-fa fa-money',
    viewModel: {
        data: {
            amount: null,
            amountUntaxed: null,
            amountPayment: null,
            _selection: null,
            lastOwnerKvid: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'OperateTime',
            direction: 'ASC'
        }]
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
            xtype: 'comboSelector',
            dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
            reference: 'depart',
            responsiveCls: 'big-100 small-100',
            keyField: 'Kvid',
            valueField: 'FullName',
            key: KiviiContext.CurrentMember.DepartmentKvid,
            value: KiviiContext.CurrentMember.DepartmentName,
            editable: false,
            width: 300,
            allowBlank: true,
            // triggers: {
            //     clear: {
            //         weight: 2,
            //         cls: 'x-fa fa-times',
            //         handler: '_cleanSelect'
            //     }
            // },
            listeners: {
                select: '_onSelect'
            }
        }, {
            xtype: 'combo',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'offseted',
            value: false,
            width: 120,
            editable: false,
            filterPickList: true,
            reference: 'offseted',
            store: {
                fields: ['name', 'offseted'],
                data: [{
                    name: '已开发票',
                    offseted: false
                }, {
                    name: '作废及红字',
                    offseted: true
                }]
            },
            listeners: {
                select: '_onSelectType'
            }
        }, {
            xtype: 'combo',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'payed',
            value: true,
            width: 100,
            emptyText: '--仅针对全部或借票有效--',
            editable: false,
            readOnly: false,
            filterPickList: true,
            reference: 'payed',
            store: {
                fields: ['name', 'payed'],
                data: [{
                    name: '已到账',
                    payed: true
                }, {
                    name: '未到账',
                    payed: false
                }, {
                    name: '全部',
                    payed: null
                }]
            },
            listeners: {
                select: '_onSelectPayed'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '时间范围  开始',
            format: 'Y-m-d',
            labelWidth: 90,
            width: 220,
            reference: 'startTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '结束',
            format: 'Y-m-d',
            labelWidth: 30,
            width: 160,
            reference: 'endTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth(), 0),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'searchField',
            flex: 1,
            labelWidth: 60,
            queryField: 'OwnerName,SerialNumber,PayerName',
            emptyText: '模糊搜索部门、发票编号、发票抬头'
        }, {
            xtype: 'downloadField',
            dataSource: '/Restful/Kivii.Finances.Entities.Invoice/Statistic.xlsx', //数据源
            fileName: '数据导出',
            window: {
                type: 'App.Control.Kivii.Finances.General.Template.Selector',
                title: '选择模板',
                iconCls: 'x-fa fa-file-excel-o',
                autoScroll: true,
                width: 700,
                height: 350,
                closeAction: 'hide',
                modal: true,
                layout: 'fit',
                filter: {
                    FolderPath: '/Finances/Invoice/Files',
                    TagsContains: 'Statistic'
                }
            }
        }]
    }, {
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            margin: '0 0 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '全页 开票金额: {amount} &nbsp;&nbsp;已到账: {amountPayment} &nbsp;&nbsp;<font color="red">未到账: {amountUntaxed} </font>'
            },
            fieldStyle: {
                fontSize: '14px'
            }
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center'
        }, {
            text: '发票号码',
            dataIndex: 'SerialNumber',
            width: 90
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            width: 150
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return '当页小计 ￥';
            },
            width: 200
        }, {
            xtype: 'datecolumn',
            text: '开票日期',
            format: 'Y-m-d',
            dataIndex: 'OperateTime',
            width: 110
        }, {
            text: '开票金额(含税)',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 130
        }, {
            text: '收入金额(不含税)',
            align: 'right',
            dataIndex: 'AmountUntaxed',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 140
        }, {
            text: '到账金额',
            align: 'right',
            dataIndex: 'AmountPayment',
            renderer: '_renderAmountPayment',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 130
        }, {
            xtype: 'datecolumn',
            text: '到账日期',
            format: 'Y-m-d',
            dataIndex: 'PayedTime',
            width: 110
        }, {
            text: '申请人',
            dataIndex: 'OperatorName',
            width: 90
            // }, {
            //     xtype: 'actioncolumn',
            //     text: '分摊状态',
            //     align: 'center',
            //     items: [{
            //         iconCls: 'x-fa fa-search',
            //         handler: '_onPayed'
            //     }],
            //     width: 90
        }, {
            dataIndex: 'Summary',
            text: '摘要',
            sortable: false,
            width: 230
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 100,
            sortable: false,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate',
        viewready: '_viewReady'
    },
    controller: {
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _viewReady: function() {
            let _me = this;
            let _view = _me.getView();
            let _vm = _me.getViewModel();
            let _store = _view.getStore();
            _store.on('load', function(_this, _records, _successful, _operation, _eOpts) {
                let _rtns = Ext.decode(_operation.getResponse().responseText);
                _vm.set('amount', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_rtns.TotalAmount.toFixed(2)));
                _vm.set('amountPayment', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_rtns.TotalAmountPayment.toFixed(2)));
                _vm.set('amountUntaxed', KiviiContext.Const.FinancesFunctionsMoneyToFmoney((_rtns.TotalAmount - _rtns.TotalAmountPayment).toFixed(2)));
            });
        },
        _renderPayerCompanyAddress: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerPhone;
        },
        _renderPayerRegisteredBank: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerBankAccount;
        },
        _renderAmountPayment: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.AmountPayment > 0 && _record.data.AmountPayment < _record.data.Amount) {
                _meta.style = 'background-color:#4682B4';
            }
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _onSelectType: function(_ctrl, _record) {
            let _controller = this,
                _view = _controller.getView(),
                _vm = _controller.getViewModel(),
                _refs = _controller.getReferences();
            delete _view.filter.Offseted;
            _view.filter.Offseted = _record.data.offseted;
            let _payed = _refs.payed;
            if (!_record.data.offseted) {
                _payed.setReadOnly(false);
                _payed.setValue(true);
                _view.filter.Payed = true;
            } else {
                _payed.setReadOnly(true);
                _payed.setValue(null);
                delete _view.filter.Payed;
            }
            _view.reload();
        },
        _onSelectPayed: function(_ctrl, _record) {
            let _controller = this,
                _view = _controller.getView(),
                _vm = _controller.getViewModel();
            delete _view.filter.Payed;
            _view.filter.Payed = _record.data.payed;
            _view.reload();
        },
        _onSelect: function(_ctrl, _record, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _ownerKvid = _record.data.Kvid;
            delete _view.filter.OwnerKvid;
            _view.filter.OwnerKvid = _ownerKvid;
            _view.reload();
        },
        _cleanSelect: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OwnerKvid;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.BeginTime;
            delete _view.filter.EndTime;
            _view.filter.BeginTime = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.EndTime = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        }
    }
});