﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentQuery : RestfulQuery<Payment>
    {

    }

    [Api(Description = "到账查询Ex")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentQueryEx : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public OwnerType OwnerType { get; set; } = OwnerType.Unsupport;

        public bool? IsPayee { get; set; }//查收款还是付款

        public bool? IsSplit { get; set; }//查是否已经拆分认领

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Bank, PaymentType.Cash, PaymentType.Pos));
            if (OwnerType == OwnerType.Organization) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.OrganizationKvid || o.OwnerKvid == Guid.Empty);
            if (OwnerType == OwnerType.Department) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.DepartmentKvid || o.OwnerKvid == Guid.Empty);
            if (OwnerType == OwnerType.Member) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid || o.OwnerKvid == Guid.Empty);
            if (IsSplit != null)
            {
                if (IsSplit.Value) sqlExpress.And(o => o.AmountSplited > 0 && o.AmountSplited == o.Amount);
                else sqlExpress.And(o => o.AmountSplited == 0 || o.AmountSplited < o.Amount);
            }
            if (IsPayee != null)
            {
                var queryAccount = conn.From<Account>();
                queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
                queryAccount.Select(o => o.Kvid);

                if (IsPayee.Value)//收款
                {
                    sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
                }
                else sqlExpress.And(o => Sql.In(o.PayeeAccountKvid, queryAccount));
            }

            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

    [Api(Description = "到账收款登记查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentPayQuery : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Split, PaymentType.Cash, PaymentType.Pos));
            sqlExpress.And(o => o.AmountUsed == 0 || o.AmountUsed < o.Amount);
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

    [Api(Description = "到账开票查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentInvoiceApplyQuery : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        //public bool Group { get; set; }//合并分组的

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Cash, PaymentType.Pos, PaymentType.Split));
            sqlExpress.And(o => o.AmountInvoice <= 0 || o.AmountInvoice < o.Amount);

            var queryInvoiceApplyRelations = conn.From<InvoiceApply>();
            queryInvoiceApplyRelations.Where(o => o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty);
            queryInvoiceApplyRelations.Select(o => o.BizKvid);
            sqlExpress.And(o => !Sql.In(o.Kvid, queryInvoiceApplyRelations));

            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

    [Api(Description = "到账冲销查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentInvoiceDebitQuery : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Split, PaymentType.Cash, PaymentType.Pos) && o.AmountInvoice < o.Amount);

            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            return rtns;
        }
    }

    [Api(Description = "作废查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentOffsetQuery : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OffsetKvid != Guid.Empty && o.Amount >= 0 && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Bank, PaymentType.Cash, PaymentType.Pos));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

    [Api(Description = "认领查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentSplitQuery : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public OwnerType OwnerType { get; set; } = OwnerType.Unsupport;

        public bool? IsBiz { get; set; }//Split,Pos,Cash,WeChat..
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            if (OwnerType == OwnerType.Organization) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.OrganizationKvid || o.OwnerKvid == Guid.Empty);
            if (OwnerType == OwnerType.Department) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.DepartmentKvid || o.OwnerKvid == Guid.Empty);
            if (OwnerType == OwnerType.Member) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid || o.OwnerKvid == Guid.Empty);
            if (IsBiz == null) sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Split, PaymentType.Refund, PaymentType.UnBiz));
            else if (IsBiz.Value) sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Split));
            else sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.Refund, PaymentType.UnBiz));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

    [Api(Description = "使用查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentUsedQuery : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => Sql.In(o.Type, PaymentType.Transfer, PaymentType.DynamicPay));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }
}
